//definition and settings
#define F_CPU 16000000UL
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <avr/sleep.h>
// serial communication library
#include "uart.h"
#define begin {
#define end }
#define t1 800
#define t2 40
#define PERIOD 200
#define HALFPER PERIOD/2
#define FULL_ROUND 1200
#define MEDIUM (delta<60)&&(delta>=30)
#define HIGH   (delta<30)&&(delta>=15)
#define LOW    (delta>=60)&&(delta<=160)
#define CLOCKWISE 0
#define COUNTERCLOCKWISE 1
#define READ(U, N) ((U) >> (N) & 1u)
#define SET(U, N) ((void)((U) |= 1u << (N)))
#define CLR(U, N) ((void)((U) &= ~(1u << (N))))
#define FLIP(U, N) ((void)((U) ^= 1u << (N)))

void motor_synchronizm(int angle, int direction);
void motor_asynchronizm(int angle, int direction);
void motor_left(int angle, int direction);
void motor_right(int angle, int direction);
void initialize(void);

volatile unsigned int time_count1,time_count2,time0, time1, time2, time3, time4, time5, time6, time7;
int delta, j, excite_flag, maxsum, minsum, irand;
unsigned char Max, Min, Max_temp, Min_temp;
unsigned char sample;

FILE uart_str = FDEV_SETUP_STREAM(uart_putchar, uart_getchar, _FDEV_SETUP_RW);

//use timer0 to build 0.05ms period
ISR (TIMER0_COMPA_vect)
begin      
  //Decrement the three times if they are not already zero
  if (time_count1>0)	--time_count1;
  if (time_count2>0)	--time_count2;
  if (time0<PERIOD)	++time0;
  if (time1<PERIOD) ++time1;
  if (time2<PERIOD)	++time2;
  if (time3<PERIOD)	++time3;
  if (time4<PERIOD)	++time4;
  if (time5<PERIOD) ++time5;
  if (time6<PERIOD)	++time6;
  if (time7<PERIOD)	++time7;
end  
// set motor to different mode according to the input signal
int main(void)
begin
	initialize();
	while(1)
	begin
		if(j<25)
		begin
			if (time_count2 == 0)
			begin
				time_count2 = t2;
				while (ADCSRA & (1<<ADSC));
				sample = ADCH;
				ADCSRA |= (1<<ADSC);
				if(sample > Max_temp) Max_temp = sample;
				if(sample < Min_temp) Min_temp = sample;
				
			end
			if (time_count1 == 0)
			begin
				time_count1 = t1;
				maxsum += Max_temp;
				minsum += Min_temp;
				Max_temp = 0;
				Min_temp = 255;
				j++;
			end
		end
		else 
		begin
			irand = rand()%10;
			j = 0;
			Max = maxsum/25;
			Min = minsum/25;
			delta = Max - Min;
			fprintf(stdout,"Max = %d, Min = %d, delta = %d, irand = %d\n\r", Max, Min, delta, irand);
			
			maxsum = 0;
			minsum = 0;
			
			if (HIGH)
			begin
				SET(PORTB, 7);
				CLR(PORTB, 5);
				CLR(PORTB, 6);
				if(excite_flag == 1)
				begin
					motor_synchronizm(100, COUNTERCLOCKWISE);
					excite_flag = 0;
				end
				//random mode: make the ear move randomly when the status is stable 
				if(irand == 0)
				begin
					motor_left(75, CLOCKWISE);
					motor_left(75, COUNTERCLOCKWISE);
					motor_left(75, CLOCKWISE);
					motor_left(75, COUNTERCLOCKWISE);
				end
				else if(irand == 1)
				begin
					motor_right(75, CLOCKWISE);
					motor_right(75, COUNTERCLOCKWISE);
					motor_right(75, CLOCKWISE);
					motor_right(75, COUNTERCLOCKWISE);
				end
			end 

			else if(LOW)
			begin
				SET(PORTB, 5);
				CLR(PORTB, 6);
				CLR(PORTB, 7);
				if(excite_flag == 0)
				begin
					motor_synchronizm(100, CLOCKWISE);
					excite_flag = 1;					
				end
				
				if(irand == 0)
				begin
					motor_left(75, CLOCKWISE);
					motor_left(75, COUNTERCLOCKWISE);
					motor_left(75, CLOCKWISE);
					motor_left(75, COUNTERCLOCKWISE);
				end
				else if(irand == 1)
				begin
					motor_right(75, CLOCKWISE);
					motor_right(75, COUNTERCLOCKWISE);
					motor_right(75, CLOCKWISE);
					motor_right(75, COUNTERCLOCKWISE);
				end
			end

			else if(MEDIUM)
			begin
				SET(PORTB, 6);
				CLR(PORTB, 5);
				CLR(PORTB, 7);

				if(irand == 0)
				begin
					motor_left(75, CLOCKWISE);
					motor_left(75, COUNTERCLOCKWISE);
					motor_left(75, CLOCKWISE);
					motor_left(75, COUNTERCLOCKWISE);
				end
				else if(irand == 1)
				begin
					motor_right(75, CLOCKWISE);
					motor_right(75, COUNTERCLOCKWISE);
					motor_right(75, CLOCKWISE);
					motor_right(75, COUNTERCLOCKWISE);
				end
			end
			else 
			begin
				CLR(PORTB, 5);
				CLR(PORTB, 6);
				CLR(PORTB, 7);
			end
		end
	end
end
// initialize parameters and set TIMERs to different mode
void initialize(void)
begin
	DDRA = 0x00;
	PORTA = 0x00;
	DDRC = 0xff;
	PORTC = 0;
	DDRB = 0xff;
	PORTB = 0xff;
	ADCSRA = (1<<ADEN) | (1<<ADSC) +7;

	ADMUX = (1<<REFS0) | (1<<ADLAR) + 0b00000; //read analog signal from PINA0
	
	//init the UART -- uart_init() is in uart.c
  	uart_init();
  	stdout = stdin = stderr = &uart_str;
  	fprintf(stdout,"Starting...\n\r");
	
	// set up timer 2 for square wave with NO ISR
	//the frequency of the squarewave is 1300Hz
	OCR2A = 191 ;	
	TCCR2B =  3;	
	// set to toggle OC2A, clear on match,
	TCCR2A = (1<<COM2A0) | (1<<WGM21) ;
	// PORT D.7 is the outout of OC2A
	DDRD = (1<<PIND7) ;	
	
	//set up timer 0 for 0.05 mSec timebase 
	TIMSK0= (1<<OCIE0A);	//turn on timer 0 cmp match ISR 
	OCR0A = 99;  		//set the compare reg to 100 time ticks
	//set prescalar to divide by 64 
	TCCR0B= 2; 	
	// turn on clear-on-match
	TCCR0A= (1<<WGM01) ;
	
	sample = 0;
	j = 0;
	excite_flag = 0;

	Max = 0;
	Min = 255;
	delta = 0;
	Max_temp = 0;
	Min_temp = 255;
	maxsum = 0;
	minsum = 0;
	
	excite_flag = 0;

	// turn on all ISRs
	sei() ;
end
// spin two ears symmetrically
void motor_synchronizm(int angle, int direction)
begin
	int cycle = (int)((float)angle /360.0 * (float)FULL_ROUND);
	int i =0;
	if (direction == COUNTERCLOCKWISE)
	begin
		time0 = 0;
		time1 = HALFPER / 2;
		time2 = HALFPER;
		time3 = HALFPER *3 /2;	
		time4 = 0;
		time7 = HALFPER / 2;
		time6 = HALFPER;
		time5 = HALFPER *3 /2;	
	end 
	else
	begin
		time0 = 0;
		time3 = HALFPER / 2;
		time2 = HALFPER;
		time1 = HALFPER *3 /2;
		time4 = 0;
		time5 = HALFPER / 2;
		time6 = HALFPER;
		time7 = HALFPER *3 /2;		
	end
	while (1)
		begin
			if (time0 == 0)
			begin
				SET(PORTC, 4);			
				i++;
			end			
			if (time0 == HALFPER)
				CLR(PORTC, 4);				
			if (time0 == PERIOD)
				time0 = 0; 

			if (time1 == 0)
				SET(PORTC, 5);				
			if (time1 == HALFPER)
				CLR(PORTC, 5);				
			if (time1 == PERIOD)
				time1 = 0; 

			if (time2 == 0)
				SET(PORTC, 6);				
			if (time2 == HALFPER)
				CLR(PORTC, 6);				
			if (time2 == PERIOD)
				time2 = 0; 

			if (time3 == 0)
				SET(PORTC, 7);			
			if (time3 == HALFPER)
				CLR(PORTC, 7);				
			if (time3 == PERIOD)
				time3 = 0; 
				
			if (time4 == 0)
			begin
				SET(PORTC, 0);				
			end			
			if (time4 == HALFPER)
				CLR(PORTC, 0);				
			if (time4 == PERIOD)
				time4 = 0; 

			if (time5 == 0)
				SET(PORTC, 1);				
			if (time5 == HALFPER)
				CLR(PORTC, 1);				
			if (time5 == PERIOD)
				time5 = 0; 

			if (time6 == 0)
				SET(PORTC, 2);				
			if (time6 == HALFPER)
				CLR(PORTC, 2);				
			if (time6 == PERIOD)
				time6 = 0; 

			if (time7 == 0)
				SET(PORTC, 3);			
			if (time7 == HALFPER)
				CLR(PORTC, 3);				
			if (time7 == PERIOD)
				time7 = 0; 

			if (i >= cycle) break;
		end
end
// spin two ears asymmetrically
void motor_asynchronizm(int angle, int direction)
begin
	int cycle = (int)((float)angle /360.0 * (float)FULL_ROUND);
	int i =0;
	if (direction == COUNTERCLOCKWISE)
	begin
		time0 = 0;
		time1 = HALFPER / 2;
		time2 = HALFPER;
		time3 = HALFPER *3 /2;	
		time4 = 0;
		time5 = HALFPER / 2;
		time6 = HALFPER;
		time7 = HALFPER *3 /2;	
	end 
	else
	begin
		time0 = 0;
		time3 = HALFPER / 2;
		time2 = HALFPER;
		time1 = HALFPER *3 /2;
		time4 = 0;
		time7 = HALFPER / 2;
		time6 = HALFPER;
		time5 = HALFPER *3 /2;		
	end
	while (1)
		begin
			if (time0 == 0)
			begin
				SET(PORTC, 4);			
				i++;
			end			
			if (time0 == HALFPER)
				CLR(PORTC, 4);				
			if (time0 == PERIOD)
				time0 = 0; 

			if (time1 == 0)
				SET(PORTC, 5);				
			if (time1 == HALFPER)
				CLR(PORTC, 5);				
			if (time1 == PERIOD)
				time1 = 0; 

			if (time2 == 0)
				SET(PORTC, 6);				
			if (time2 == HALFPER)
				CLR(PORTC, 6);				
			if (time2 == PERIOD)
				time2 = 0; 

			if (time3 == 0)
				SET(PORTC, 7);			
			if (time3 == HALFPER)
				CLR(PORTC, 7);				
			if (time3 == PERIOD)
				time3 = 0; 
				
			if (time4 == 0)
			begin
				SET(PORTC, 0);				
			end			
			if (time4 == HALFPER)
				CLR(PORTC, 0);				
			if (time4 == PERIOD)
				time4 = 0; 

			if (time5 == 0)
				SET(PORTC, 1);				
			if (time5 == HALFPER)
				CLR(PORTC, 1);				
			if (time5 == PERIOD)
				time5 = 0; 

			if (time6 == 0)
				SET(PORTC, 2);				
			if (time6 == HALFPER)
				CLR(PORTC, 2);				
			if (time6 == PERIOD)
				time6 = 0; 

			if (time7 == 0)
				SET(PORTC, 3);			
			if (time7 == HALFPER)
				CLR(PORTC, 3);				
			if (time7 == PERIOD)
				time7 = 0; 

			if (i >= cycle) break;
		end
end
// spin left ear
void motor_left(int angle, int direction)
begin
	int cycle = (int)((float)angle /360.0 * (float)FULL_ROUND);
	int i =0;
	if (direction == COUNTERCLOCKWISE)
	begin
		time0 = 0;
		time1 = HALFPER / 2;
		time2 = HALFPER;
		time3 = HALFPER *3 /2;	
	end
	else
	begin
		time0 = 0;
		time3 = HALFPER / 2;
		time2 = HALFPER;
		time1 = HALFPER *3 /2;
	end
	while (1)
		begin
			if (time0 == 0)
			begin
				SET(PORTC, 4);			
				i++;
			end			
			if (time0 == HALFPER)
				CLR(PORTC, 4);				
			if (time0 == PERIOD)
				time0 = 0; 

			if (time1 == 0)
				SET(PORTC, 5);				
			if (time1 == HALFPER)
				CLR(PORTC, 5);				
			if (time1 == PERIOD)
				time1 = 0; 

			if (time2 == 0)
				SET(PORTC, 6);				
			if (time2 == HALFPER)
				CLR(PORTC, 6);				
			if (time2 == PERIOD)
				time2 = 0; 

			if (time3 == 0)
				SET(PORTC, 7);			
			if (time3 == HALFPER)
				CLR(PORTC, 7);				
			if (time3 == PERIOD)
				time3 = 0; 
			
			if (i >= cycle) break;
		end				
end
// spin right ear
void motor_right(int angle, int direction)
begin
	int cycle = (int)((float)angle /360.0 * (float)FULL_ROUND);
	int i =0;
	if (direction == COUNTERCLOCKWISE)
	begin
		time4 = 0;
		time5 = HALFPER / 2;
		time6 = HALFPER;
		time7 = HALFPER *3 /2;	
	end
	else
	begin
		time4 = 0;
		time7 = HALFPER / 2;
		time6 = HALFPER;
		time5 = HALFPER *3 /2;
	end
	while (1)
		begin
			if (time4 == 0)
			begin
				SET(PORTC, 0);			
				i++;
			end			
			if (time4 == HALFPER)
				CLR(PORTC, 0);				
			if (time4 == PERIOD)
				time4 = 0; 

			if (time5 == 0)
				SET(PORTC, 1);				
			if (time5 == HALFPER)
				CLR(PORTC, 1);				
			if (time5 == PERIOD)
				time5 = 0; 

			if (time6 == 0)
				SET(PORTC, 2);				
			if (time6 == HALFPER)
				CLR(PORTC, 2);				
			if (time6 == PERIOD)
				time6 = 0; 

			if (time7 == 0)
				SET(PORTC, 3);			
			if (time7 == HALFPER)
				CLR(PORTC, 3);				
			if (time7 == PERIOD)
				time7 = 0; 
			if (i >= cycle) break;
		end
end
